#!/usr/bin/env python
# encoding: utf-8
'''
editing.filter_reads -- shortdesc

editing.filter_reads is a description

It defines classes_and_methods

@author:     user_name

@copyright:  2016 organization_name. All rights reserved.

@license:    license

@contact:    user_email
@deffield    updated: Updated
'''

import sys
import os
import pandas as pd

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter

__all__ = []
__version__ = 0.1
__date__ = '2016-07-13'
__updated__ = '2016-07-13'

DEBUG = 0
TESTRUN = 0
PROFILE = 0

class CLIError(Exception):
    '''Generic exception to raise and log different fatal errors.'''
    def __init__(self, msg):
        super(CLIError).__init__(type(self))
        self.msg = "E: %s" % msg
    def __str__(self):
        return self.msg
    def __unicode__(self):
        return self.msg

def filter_known_snp(infile, known, outfile):
    """
    Step 7: Remove known SNPs
    
    We don't want to erroneously call editing sites that are known SNPs.
    Filter these out with a BED3 file with known SNP locations (source: wormbase)
    """
    print("Filtering known SNPs: {}".format(infile))
    o = open(outfile,'w')
    with open(infile,'r') as f:
        for line in f:
            if line.startswith('#'):
                o.write(line)
    o.close()
    
    names1 = ['CHROM','POS','ID','REF','ALT','QUAL','FILTER','INFO','FORMAT','.']
    eff_df = pd.read_table(infile,comment='#',names=names1,dtype={'QUAL':str})
    
    names2 = ['CHROM','START','POS'] # POS is the 1-based position of the SNP.
    if(known.endswith('.gz')):
        snp_df = pd.read_table(known,compression='gzip',names=names2)
    else:
        snp_df = pd.read_table(known,names=names2)
    
    snp_df['KNOWN'] = 1
    joined = pd.merge(eff_df,snp_df,how='left',on=['CHROM','POS'])
    print("Number of known SNPs filtered out: {}".format(joined[joined['KNOWN']==1].shape[0]))
    keep = joined[joined['KNOWN']!=1]
    
    del keep['KNOWN']
    
    with open(outfile,'a') as o:
        keep.to_csv(o, header=False, index=False,sep='\t')
    
def main(argv=None): # IGNORE:C0111
    '''Command line options.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
    program_shortdesc = __import__('__main__').__doc__.split("\n")[1]
    program_license = '''%s

  Created by user_name on %s.
  Copyright 2016 organization_name. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, str(__date__))

    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument('-V', '--version', action='version', version=program_version_message)
        parser.add_argument("-i", "--input", dest="input_eff", help="variant eff(vcf) file for filtering", required = True)
        parser.add_argument("-o", "--output", dest="output_noSNP", help="output noSNP (vcf) file with known snps removed.", required = True)
        parser.add_argument("-k", "--known", dest="known", help="bed3 file of known snps", required = True )
        # Process arguments
        args = parser.parse_args()
        input_eff = args.input_eff
        output_noSNP = args.output_noSNP
        
        known = args.known
        
        filter_known_snp(input_eff, known, output_noSNP)
        return 0
    except KeyboardInterrupt:
        ### handle keyboard interrupt ###
        return 0
    except Exception, e:
        if DEBUG or TESTRUN:
            raise(e)
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        return 2

if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-h")
        sys.argv.append("-v")
        sys.argv.append("-r")
    if TESTRUN:
        import doctest
        doctest.testmod()
    if PROFILE:
        import cProfile
        import pstats
        profile_filename = 'editing.filter_reads_profile.txt'
        cProfile.run('main()', profile_filename)
        statsfile = open("profile_stats.txt", "wb")
        p = pstats.Stats(profile_filename, stream=statsfile)
        stats = p.strip_dirs().sort_stats('cumulative')
        stats.print_stats()
        statsfile.close()
        sys.exit(0)
    sys.exit(main())



#!/usr/bin/env python
# encoding: utf-8
'''
editing.filter_reads -- shortdesc

editing.filter_reads is a description

It defines classes_and_methods

@author:     user_name

@copyright:  2016 organization_name. All rights reserved.

@license:    license

@contact:    user_email
@deffield    updated: Updated
'''

import sys
import os
import re
import pysam

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter

__all__ = []
__version__ = 0.1
__date__ = '2016-07-13'
__updated__ = '2016-07-13'

DEBUG = 0
TESTRUN = 0
PROFILE = 0

class CLIError(Exception):
    '''Generic exception to raise and log different fatal errors.'''
    def __init__(self, msg):
        super(CLIError).__init__(type(self))
        self.msg = "E: %s" % msg
    def __str__(self):
        return self.msg
    def __unicode__(self):
        return self.msg

def filter_reads(input_bam, output_bam, min_overhang, min_underhang, non_ag_mm_threshold):
    """
    # Step 3: filter reads
    
    Per Boyko/Mike Washburn's editing paper: 
    
    b) it had a junction overhang < 10nt according to its SAMtools CIGAR string
    c) it had > 1 non-A2G or non-C2T mismatch or any short indel, per its MD tag
    d) it had a mismatch less than 25nt away from either end of the read
    (this was changed to 5nt in the relaxed version used for quantification)
    
    * Removed 5d per discussions with the Hundley lab.
    """
    print("Filtering reads on: {}".format(input_bam))
    i = pysam.AlignmentFile(input_bam)
    o = pysam.AlignmentFile(output_bam, "wb", template=i)
    for read in i:
        try:
            flag = 1
            
            """
            Throw out unmapped reads
            """
            if read.is_unmapped:
                flag = 2
                continue
                # print(read)
            non_ag_mm_counts = 0
            try:
                mm = read.get_tag('MD')
            except KeyError:
                mm = ''
                # print(read.query_name)
            cigar = read.cigarstring
            read_seq = read.query_sequence
            
            """
            Takes care of soft clipped bases
            (remove bases from the read_seq which are soft clipped
            to not interfere with mis-alignments downstream)
            """
            softclip_regex = ur"^(\d+)S"
            softclip = re.findall(softclip_regex,cigar)
            if(softclip):
                if (len(softclip) == 2):
                    softclip2 = softclip[1]
                    read_seq = read_seq[:-(int(softclip2))] # remove softclipped reads
                softclip1 = softclip[0]
                read_seq = read_seq[int(softclip1):] # remove softclipped reads
                    
            """
            # 5b) Check for small junction overhangs. If junction over hang is small, remove read
            """
            cigar_overhang_regex = ur"^(\d+)M.*[\D](\d+)M$"
            
            #print("cigar: {}".format(line[5]))
            overhangs = re.findall(cigar_overhang_regex,cigar)
                    
            if(overhangs):
                # print("cigar: {}, overhangs: {}".format(cigar,overhangs))
                overhangs = overhangs[0]
                #print(overhangs)
                if(len(overhangs) == 1):
                    if(int(overhangs[0]) < min_overhang):
                        # print("overhang 1 does not meet the min overhang reqs",overhangs[0])
                        # print(read)
                        flag = 3
                elif(len(overhangs) == 2):
                    if ((int(overhangs[1]) < min_overhang) or (int(overhangs[0]) < min_overhang)):
                        # print("overhang 2 does not meet the min overhang reqs",overhangs[1])
                        # print(read)
                        flag = 4
                    
            """
            # 5c) If there exists short indels, remove them.
            """
            if('I' in cigar or 'D' in cigar):
                # print('filtered out indels (cigar={})'.format(cigar))
                flag = 5
                # print(read)
            
            """
            # 2) If primary not primary alignment, throw out
            """
    
            if(read.is_secondary):
                flag = 6
                # print(read)
            """
            # MD:Z tag-based filters
            """
            flank_mm_regex = ur"^(\d+).*[ACGT](\d+)$"
            flank_mm = re.findall(flank_mm_regex,mm)
            if(flank_mm):
                flank_mm = flank_mm[0]
                if(flank_mm[1]):
                    if(int(flank_mm[1]) < min_underhang):
                        print("{} does not meet min underhang.".format(flank_mm[1]))
                        flag = 7
                if(flank_mm[0]):
                    if(int(flank_mm[0]) < min_underhang):
                        print("{} does not meet min underhang.".format(flank_mm[0]))
                        flag = 8
            
            """
            # Manually setting reversed reads to 'sense' strand per truseq library protocols
            """
            sense = True if read.is_reverse == True else False
            
            """
            # 5c) Search MDZ for A, T's in reference, if mutations are not 
            #     A-G (sense) or T-C (antisense), add to non_ag_mm_counts threshold.
            #     If non_ag_mm_counts > threshold, toss the whole read. Otherwise,
            #     allow up to the maximum allowable non-AG mismatches before tossing.
            #     Default: 1 mm
            """
            mismatches_regex = ur"(\d+)([ATCG])"
            mismatches = re.findall(mismatches_regex,mm)
            if(mismatches):
                
                read_pos = 0
                non_ag_mm_counts = 0
                for mismatch in mismatches:
                    ref_allele = mismatch[1]
                    read_pos = read_pos + int(mismatch[0])
                                
                    read_allele = read_seq[read_pos]
                        
                    if(not((ref_allele == 'A' and read_allele == 'G' and sense == True) or \
                           (ref_allele == 'T' and read_allele == 'C' and sense == False))):
                        non_ag_mm_counts = non_ag_mm_counts + 1
                        if non_ag_mm_counts > non_ag_mm_threshold:
                            flag = 9
                            break
                    read_pos = read_pos + 1
                
                    
                    # print(read)
           
            if flag == 1:
                o.write(read)
        except TypeError as e:
            print("error! {}".format(e))
            print(read)
    i.close()
    o.close()
    
def main(argv=None): # IGNORE:C0111
    '''Command line options.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
    program_shortdesc = __import__('__main__').__doc__.split("\n")[1]
    program_license = '''%s

  Created by user_name on %s.
  Copyright 2016 organization_name. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, str(__date__))

    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument('-V', '--version', action='version', version=program_version_message)
        parser.add_argument("-i", "--input", dest="input_bam", help="sorted rmdup bam file for filtering", required = True)
        parser.add_argument("-o", "--output", dest="output_bam", help="output bam filtered per editing rules", required = True)
        parser.add_argument("-j", "--junction_overhang", dest="min_overhang", help="minimum junction overhang needed to start counting mutations", required = False, type=int, default = 10 )
        parser.add_argument("-e", "--edge_mutation", dest="min_underhang", help="minimum nt away from read end to call valid mutation", required = False, type=int, default = 0 )
        parser.add_argument("-ag", "--non_ag_threshold", dest="non_ag_mm_threshold", help="allow up to [n] non-AG mutations in a read before tossing", required = False, type=int, default = 1 )
        # Process arguments
        args = parser.parse_args()
        input_bam = args.input_bam
        output_bam = args.output_bam
        min_overhang = args.min_overhang
        min_underhang = args.min_underhang
        non_ag_mm_threshold = args.non_ag_mm_threshold
        
        filter_reads(input_bam, output_bam, min_overhang, min_underhang, non_ag_mm_threshold)
        return 0
    except KeyboardInterrupt:
        ### handle keyboard interrupt ###
        return 0
    except Exception, e:
        if DEBUG or TESTRUN:
            raise(e)
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        return 2

if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-h")
        sys.argv.append("-v")
        sys.argv.append("-r")
    if TESTRUN:
        import doctest
        doctest.testmod()
    if PROFILE:
        import cProfile
        import pstats
        profile_filename = 'editing.filter_reads_profile.txt'
        cProfile.run('main()', profile_filename)
        statsfile = open("profile_stats.txt", "wb")
        p = pstats.Stats(profile_filename, stream=statsfile)
        stats = p.strip_dirs().sort_stats('cumulative')
        stats.print_stats()
        statsfile.close()
        sys.exit(0)
    sys.exit(main())


#!/usr/bin/env python
# encoding: utf-8
'''
editing.filter_reads -- shortdesc

editing.filter_reads is a description

It defines classes_and_methods

@author:     user_name

@copyright:  2016 organization_name. All rights reserved.

@license:    license

@contact:    user_email
@deffield    updated: Updated
'''

import sys
import os
import re
import pysam

from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter

__all__ = []
__version__ = 0.1
__date__ = '2016-07-13'
__updated__ = '2016-07-13'

DEBUG = 0
TESTRUN = 0
PROFILE = 0

class CLIError(Exception):
    '''Generic exception to raise and log different fatal errors.'''
    def __init__(self, msg):
        super(CLIError).__init__(type(self))
        self.msg = "E: %s" % msg
    def __str__(self):
        return self.msg
    def __unicode__(self):
        return self.msg

def vcf2eff(input_vcf, output_eff, min_coverage):
    """
    Step 6: Filter variants for coverage and editing-specific AG/TC changes.
    
    Converts vcf file into an "eff" file (essentially filtering for variants at this step).
    
    """
    print("Calling vcf2eff: {}".format(input_vcf))
    o = open(output_eff,'w')
    with open(input_vcf) as f:
        # for line in f:
        for line in f:
            flag = 1 # if flag remains at 1 throughout filter, this is a 'good' read to keep
            
            if line.startswith('#'): # is vcf header
                o.write(line)
            else:
                line = line.split('\t')
                ref = line[3]
                alt = line[4]
                
                dp, i = re.findall('DP\=(\d+)\;.*DP4\=([\d\,]+)', line[7])[0]
                """
                if the DP flag does not meet minimum coverage, toss it.
                """
                i = i.split(',')
                if(int(dp) < min_coverage):
                    flag = 0
                
                """
                i[0] = forward ref allele
                i[1] = reverse ref allele
                i[2] = forward non-ref allele
                i[3] = reverse non-ref allele
                
                logic: if there are more ref/non-ref alleles on the reverse strand, 
                is it 'reversed' due to TruSeq stranded preparation.
                
                """
                
                
                if((int(i[0]) + int(i[2])) < (int(i[1]) + int(i[3]))): 
                    sense = True
                    refNum = int(i[1])
                    altNum = int(i[3])
                else: 
                    sense = False
                    refNum = int(i[0])
                    altNum = int(i[2])
                
                """
                Get edit percentage 
                """
                if(altNum + refNum) > 0:
                    line[5] = str(altNum / float(altNum + refNum))
                    line[2] = str(altNum + refNum)
                    
                    eff = re.findall('EFF=([^\(]+)',line[7])
                    if len(eff) > 0:
                        line[6] = eff # post filtering step after snpEff
                
                """
                # 6) if not A-G or T-C, then pass
                
                """
                if((ref == 'A' and alt.find('G')==-1) or (ref == 'T' and alt.find('C')==-1)):
                    flag = 0
                    
                """
                # 6) if reference isn't A (forward) or T (reverse), toss it
                """
                if ((sense and ref != 'A') or (not sense and ref != 'T')) :
                    flag = 0
                
                """
                if vcf line survives all filters, write it.
                """
                if flag == 1:
                    newline = "\t".join([l for l in line])
                    o.write(newline)
    o.close()
    
def main(argv=None): # IGNORE:C0111
    '''Command line options.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
    program_shortdesc = __import__('__main__').__doc__.split("\n")[1]
    program_license = '''%s

  Created by user_name on %s.
  Copyright 2016 organization_name. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, str(__date__))

    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument('-V', '--version', action='version', version=program_version_message)
        parser.add_argument("-i", "--input", dest="input_vcf", help="variant vcf file for filtering", required = True)
        parser.add_argument("-o", "--output", dest="output_eff", help="output vcf (eff) filtered per editing rules", required = True)
        parser.add_argument("-m", "--min_coverage", dest="min_coverage", help="minimum allowable read coverage to pass variant filter", required = False, type=int, default = 10 )
        # Process arguments
        args = parser.parse_args()
        input_vcf = args.input_vcf
        output_eff = args.output_eff
        
        min_coverage = args.min_coverage
        
        vcf2eff(input_vcf, output_eff, min_coverage)
        return 0
    except KeyboardInterrupt:
        ### handle keyboard interrupt ###
        return 0
    except Exception, e:
        if DEBUG or TESTRUN:
            raise(e)
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        return 2

if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-h")
        sys.argv.append("-v")
        sys.argv.append("-r")
    if TESTRUN:
        import doctest
        doctest.testmod()
    if PROFILE:
        import cProfile
        import pstats
        profile_filename = 'editing.filter_reads_profile.txt'
        cProfile.run('main()', profile_filename)
        statsfile = open("profile_stats.txt", "wb")
        p = pstats.Stats(profile_filename, stream=statsfile)
        stats = p.strip_dirs().sort_stats('cumulative')
        stats.print_stats()
        statsfile.close()
        sys.exit(0)
    sys.exit(main())



#!/usr/bin/env python
# encoding: utf-8
'''
editing.filter_reads -- shortdesc

editing.filter_reads is a description

It defines classes_and_methods

@author:     user_name

@copyright:  2016 organization_name. All rights reserved.

@license:    license

@contact:    user_email
@deffield    updated: Updated
'''

import sys
import os
import pybedtools as bt
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter

__all__ = []
__version__ = 0.1
__date__ = '2016-07-13'
__updated__ = '2016-07-13'

DEBUG = 0
TESTRUN = 0
PROFILE = 0

class CLIError(Exception):
    '''Generic exception to raise and log different fatal errors.'''
    def __init__(self, msg):
        super(CLIError).__init__(type(self))
        self.msg = "E: %s" % msg
    def __str__(self):
        return self.msg
    def __unicode__(self):
        return self.msg

def is_intergenic(row):
    """
    A genic region is defined as either: 
        1) an intron
        2) an exon (5' UTR, CDS, 3' UTR)
    is_intergenic determines if a row (from a bedtools closest result) 
    lies within genic regions. Returns the region name if genic, otherwise
    returns 'intergenic|whatever the closest genic region is'
    """
    if row['distance_from_region'] > 0:
        return "intergenic|{}".format(row['region_name'])
    else:
        return row['region_name']

def priority(row):
    """
    If there are overlapping annotations in the gff file, we will prioritize 
    regions our SNV is contained in:
        1) 3' UTR
        2) 5' UTR
        3) CDS
        4) Intron
    Returns an assigned priority 
    """
    if row['region'] == 'three_prime_UTR':
        return 1
    elif row['region'] == 'five_prime_UTR':
        return 2
    elif row['region'] == 'CDS':
        return 3
    elif row['region'] == 'Intron':
        return 4

def conf_to_bed(conf_file, output_bedfile, min_confidence):
    """
    Step 9: Converts a "conf" file (vcf format) into a bedfile.
    """
    print("Converting conf files to bedfiles: {}".format(conf_file))
    bedfile = open(output_bedfile,'w')
    with open(conf_file,'r') as f:
        for line in f:
            strand = '.'
            if not line.startswith('#'):
                line = line.split('\t')
                if line[3] == 'A':
                    strand = '+'
                elif line[3] == 'T':
                    strand = '-'
                if float(line[5]) >= min_confidence:
                    bedfile.write("{0}\t{1}\t{2}\t{3:.2f}\t{4}\t{5}\n".format(line[0],
                                                                          int(line[1])-1,
                                                                          int(line[1]),
                                                                          int(float(line[7])*100),
                                                                          line[2],
                                                                          strand))
    bedfile.close()

def annotate_bed(input_bedfile, output_bedfile, gff3):
    
    """
    Step 10: Given a bedfile and annotation gff3, annotate regions.
    """
    print("Annotating: {}".format(input_bedfile))
    genes = bt.BedTool(gff3)
    conf = bt.BedTool(input_bedfile)
    annotations = conf.closest(genes, d=True).to_dataframe(names=['chrom',
                                                                  'pos-1',
                                                                  'pos',
                                                                  'approx_edit_fraction',
                                                                  'approx_coverage',
                                                                  'strand',
                                                                  'region_chrom',
                                                                  'source',
                                                                  'region_name',
                                                                  'region_start',
                                                                  'region_end',
                                                                  'region_.',
                                                                  'region_strand',
                                                                  '.',
                                                                  'region_annotation',
                                                                  'distance_from_region'])
    annotations['region'] = annotations.apply(is_intergenic,axis=1)
    del annotations['region_name'] # delete this as 'region' effectively replaces it.
    annotations['priority'] = annotations.apply(priority,axis=1)
    annotations.sort_values(by=['priority'])
    annotations.drop_duplicates(subset=['chrom',
                                        'pos-1',
                                        'pos',
                                        'approx_coverage',
                                        'approx_edit_fraction',
                                        'strand'],
                                inplace=True)
    del annotations['priority'] # not part of a proper bedfile
    del annotations['region_.'] # not part of a proper bedfile
    del annotations['.'] # not part of a proper bedfile
    annotations['region_annotation'] = annotations['region_annotation'].str.replace('Parent=Transcript:','')
    # annotations = annotations[annotations['pos'].isin(falsepositives)==False]
    annotations.to_csv(output_bedfile,sep="\t",header=True,index=None)
      
    
def main(argv=None): # IGNORE:C0111
    '''Command line options.'''

    if argv is None:
        argv = sys.argv
    else:
        sys.argv.extend(argv)

    program_name = os.path.basename(sys.argv[0])
    program_version = "v%s" % __version__
    program_build_date = str(__updated__)
    program_version_message = '%%(prog)s %s (%s)' % (program_version, program_build_date)
    program_shortdesc = __import__('__main__').__doc__.split("\n")[1]
    program_license = '''%s

  Created by user_name on %s.
  Copyright 2016 organization_name. All rights reserved.

  Licensed under the Apache License 2.0
  http://www.apache.org/licenses/LICENSE-2.0

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
''' % (program_shortdesc, str(__date__))

    try:
        # Setup argument parser
        parser = ArgumentParser(description=program_license, formatter_class=RawDescriptionHelpFormatter)
        parser.add_argument('-V', '--version', action='version', version=program_version_message)
        parser.add_argument("-i", "--input", dest="conf", help="confidence (vcf) file for conversion to bed", required = True)
        parser.add_argument("-o", "--output", dest="bed", help="output bedfile", required = True)
        parser.add_argument("-c", "--confidence", dest="confidence", help="confidence threshold cutoff", required = False, default = 0.995, type = float )
        parser.add_argument("-g", "--gff3", dest="gff3", help="gff3 annotation file", required = True )
        
        # Process arguments
        args = parser.parse_args()
        conf_file = args.conf
        bedfile = args.bed
        min_confidence = args.confidence
        gff3 = args.gff3
                
        conf_to_bed(conf_file, bedfile.replace('.bed','.unannotated.bed'), min_confidence)
        annotate_bed(bedfile.replace('.bed','.unannotated.bed'), bedfile, gff3)
        return 0
    except KeyboardInterrupt:
        ### handle keyboard interrupt ###
        return 0
    except Exception, e:
        if DEBUG or TESTRUN:
            raise(e)
        indent = len(program_name) * " "
        sys.stderr.write(program_name + ": " + repr(e) + "\n")
        sys.stderr.write(indent + "  for help use --help")
        return 2

if __name__ == "__main__":
    if DEBUG:
        sys.argv.append("-h")
        sys.argv.append("-v")
        sys.argv.append("-r")
    if TESTRUN:
        import doctest
        doctest.testmod()
    if PROFILE:
        import cProfile
        import pstats
        profile_filename = 'editing.filter_reads_profile.txt'
        cProfile.run('main()', profile_filename)
        statsfile = open("profile_stats.txt", "wb")
        p = pstats.Stats(profile_filename, stream=statsfile)
        stats = p.strip_dirs().sort_stats('cumulative')
        stats.print_stats()
        statsfile.close()
        sys.exit(0)
    sys.exit(main())
